/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <clack/c/packet.h>
#include <clack/c/packet_ip.h>
#include <clack/c/packet_icmp.h>
#include <netinet/ip.h>
#include <assert.h>

extern int ttl;
//static const int payload_size = 0;  // We'll add 8 to this number

// ToDo: we hack in the number 56 to make the packet big enough
// for an ethernet packet and also to make it the same size as the
// usual size of echo requests.
static const int payload_size = 56;  // We'll add 8 to this number

clack_packet_t* rfun(clack_packet_t *request)
{
        struct ip      *request_ip   = ((struct ip*)clack_packet_data(request));
        struct icmphdr *request_icmp = ((struct icmphdr*)((void*)request_ip+(request_ip->ip_hl << 2)));
        clack_packet_t *reply = clack_packet_alloc_padded(payload_size, sizeof (struct ip) + sizeof (struct icmphdr));
        assert(reply); // ToDo: kill all asserts

        memcpy(clack_packet_data(reply), request_icmp+1, 8 + payload_size);

        add_icmp_hdr_echo(reply, ICMP_ECHOREPLY, 0, request_icmp->un.echo.id, request_icmp->un.echo.sequence);
        add_ip_hdr(reply, IPPROTO_ICMP, ttl, request_ip->ip_dst, request_ip->ip_src);  // swap src/dst
        
        return reply;
}
