/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <clack/c/packet.h>
#include <clack/c/packet_eth.h>
#include <clack/c/packet_ip.h>
#include <clack/c/packet_icmp.h>
#include <netinet/ip.h>
#include <assert.h>

extern int ttl;
extern clack_size_t size;

clack_packet_t* spawn(clack_packet_t *orig)
{
        struct ip      *orig_ip      = (struct ip*)clack_packet_data(orig);
        clack_size_t    orig_ip_hl   = orig_ip->ip_hl << 2;
        clack_size_t    payload_size = orig_ip_hl + 8;

        clack_packet_t *p = clack_packet_alloc_padded(payload_size, sizeof (struct ip) + sizeof (struct icmphdr));
        assert(p); // ToDo: kill all asserts

        memcpy(clack_packet_data(p), clack_packet_data(orig), payload_size);
        add_icmp_hdr_frag(p,ICMP_DEST_UNREACH,ICMP_FRAG_NEEDED,size);
        add_ip_hdr(p,IPPROTO_ICMP,ttl,orig_ip->ip_dst,orig_ip->ip_src); // swap src/dst
        p->anno.fix_ip_src = 1;

        return p;
}
