/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

/*
 * ipinputcombo, which corresponds to the Click combined element used
 * to replace:
 *
 *	--> Strip(14) --> CheckIPHeader(...) --> GetIPAddress(16) -->
 *
 * although the Click version also includes painting the color.
 */

#include <clack/c/packet.h>
#include <clack/c/packet_ip.h>
#include <arpa/inet.h>
#include <assert.h>
#include <stdlib.h>

/*
 * Imported
 */
extern clack_error_t	push_toip(clack_packet_t *);
extern clack_error_t	push_tofail(clack_packet_t *);
extern int		n_bad_src;
extern char		*bad_src[];

/*
 * Exported
 */
clack_error_t		push(clack_packet_t *p);

/*
 * The converted list of illegal addresses.
 */
static struct in_addr   *badaddrs;

clack_error_t
init(void)
{
	int i;
  
	badaddrs = malloc(n_bad_src * sizeof(struct in_addr));
	assert(badaddrs);

	for (i = 0; i < n_bad_src; i++)
		inet_aton(bad_src[i], &badaddrs[i]);

	return 0;
}

clack_error_t
push(clack_packet_t *p)
{
	struct ip	*ip;
	clack_size_t	length;
	unsigned	hlen;
	int		i;

	/* Strip(14) */
        clack_packet_shift(p, 14);

	ip     = (struct ip *) clack_packet_data(p);
	length = clack_packet_length(p);
	
	if (length < sizeof(struct ip))
		goto bad;
	if (ip->ip_v != IPVERSION)
		goto bad;
  
	hlen = ip->ip_hl << 2;
	if (hlen < sizeof(struct ip))
		goto bad;
	if (hlen > length)
		goto bad;
  
	if (in_cksum((unsigned char *)ip, hlen) != 0)
		goto bad;
	if (ntohs(ip->ip_len) < hlen)
		goto bad;

	/*
	 * RFC1812 5.3.7 and 4.2.2.11: discard illegal source addresses.
	 * Configuration string should have listed all subnet
	 * broadcast addresses known to this router.
	 */
	for(i = 0; i < n_bad_src; i++) {
		if (ip->ip_src.s_addr == badaddrs[i].s_addr)
			goto bad;
	}

	/* GetIPAddress */
        p->anno.next_hop = ip->ip_dst;
	
	push_toip(p);
	return 0;
 bad:
	push_tofail(p);
	return 0;
}
